import logging
import os
from abc import ABC

from notification import constants
from notification.base_notifier import BaseNotifier
import psc_orgkey
import urllib.parse

LOG_LEVEL = os.environ.get('log_level', 'info').upper()
logger = logging.getLogger()
logger.setLevel(logging.getLevelName(LOG_LEVEL))


class CspNotifier(BaseNotifier, ABC):
    def __init__(self, email_subject, email_sender, email_sender_name, message_type, env, client, template,
                 wavefront_registry, url_mappings, external_ingress_hostname, telegraf_client=None):
        super().__init__(url_mappings, wavefront_registry, telegraf_client=None)
        self.message_type = message_type
        self.email_subject = email_subject
        self.email_sender = email_sender
        self.email_sender_name = email_sender_name
        self.env = env
        self.csp_client = client
        self.template = template
        self.external_ingress_hostname = external_ingress_hostname
    def send(self, event):
        try:
            self.csp_client.login()
            publisher = self.csp_client.get_publisher()
            if not publisher:
                publisher_id = self.csp_client.create_publisher(self.message_type)
            else:
                publisher_id = publisher['publisherID']
                if not self.csp_client.message_type_exist(publisher, self.message_type):
                    self.csp_client.add_message_type(publisher_id, self.message_type)

            template_group_id = self.csp_client.get_template_group_id(publisher_id, self.message_type)
            if not template_group_id:
                self.csp_client.create_template_group(
                    publisher_id, self.message_type, constants.TEMPLATE_NAME, self.email_subject,
                    self.email_sender_name, self.email_sender, self.template)

            org_key = psc_orgkey.decode_orgkey(event['orgKey'])
            base_url = self.get_base_url_for_env(cbd_env=org_key.env, external_ingress_hostname=self.external_ingress_hostname)
            substitutions = self.prepare_substitution_data(base_url=base_url,
                                                           started_millis=event['startTime']['epochSecond'],
                                                           status=event['status'],
                                                           user=event['admin'],
                                                           query_name=(event.get('name', None) or '').strip() or None,
                                                           query_id=event['queryId'])
            message_id = self.csp_client.send_email(publisher_id, self.message_type, event['admin'], substitutions)
            self.wavefront_counter('LIVE_QUERY.notifications')
            return message_id
        except Exception as e:
            logger.error('Failed to send notification. Response = {}'.format(e))
            self.wavefront_counter('LIVE_QUERY.failed_notifications')
            raise e

    def prepare_substitution_data(self, base_url, started_millis, status, user, query_name, query_id):
        substitutions = {
            "base_url": base_url.rstrip('/'),
            "status": status.replace("_", " ").title(),
            "name": query_name,
            "user": user,
            "query_id": query_id,
            "time": self.format_timestamp(started_millis)
        }
        return substitutions

    @classmethod
    def get_env_value_from_mapping(cls, mapping, cbd_env: str, csp_endpoint):
        cbd_env = cbd_env.lower()
        value = mapping.get(cbd_env, None)
        if value:
            return value
        elif csp_endpoint:
            parsed_url = urllib.parse.urlparse(csp_endpoint)
            return parsed_url.netloc
        raise ValueError('Unsupported environment: {env}'.format(env=cbd_env))
