import logging
import os
from abc import ABC

import psc_orgkey

from notification import metrics
from notification.base_notifier import BaseNotifier

LOG_LEVEL = os.environ.get('log_level', 'info').upper()
logger = logging.getLogger()
logger.setLevel(logging.getLevelName(LOG_LEVEL))
TELEGRAF_REPORTER_ENABLED = os.environ.get('TELEGRAF_REPORTER_ENABLED', True)
ENV = os.environ.get('ENV', 'local')
SERVICE_NAME = 'livequery-notifier'
HOSTNAME = os.environ.get('HOSTNAME', 'localhost-notifier')
REGION = os.environ.get('region', 'us-east-1')


class SesNotifier(BaseNotifier, ABC):
    def __init__(self, email_subject, email_sender, env, client, template, wavefront_registry, url_mappings, telegraf_client):
        """

        :param email_subject:
        :param email_sender:
        :param env:
        :param client:
        :param template:
        :param wavefront_registry:
        :param url_mappings:
        """
        super().__init__(url_mappings, wavefront_registry, telegraf_client)
        self.email_subject = email_subject
        self.email_sender = email_sender
        self.env = env
        self.client = client
        self.template = template

    def send(self, event):
        """

        :param event: notification event
        :return:
        """
        org_key = psc_orgkey.decode_orgkey(event['orgKey'])
        base_url = self.get_base_url_for_env(cbd_env=org_key.env, external_ingress_hostname=None)
        template_data = self.prepare_data(base_url=base_url, admin=event['admin'],
                                          started_millis=event['startTime']['epochSecond'],
                                          status=event['status'].replace("_", " ").title(),
                                          query_name=(event.get('name', None) or '').strip() or None,
                                          query_id=event['queryId'])
        email_subject = self.email_subject
        if template_data['name']:
            email_subject += f" - {template_data['name']}"

        result = self.client.send_email(**SesNotifier
                                        .get_email_payload(email_subject=self.email_subject,
                                                           email_sender=self.email_sender,
                                                           admin=event['admin'], template_data=template_data,
                                                           template=self.template))

        if result and 'ResponseMetadata' in result and result['ResponseMetadata']['HTTPStatusCode'] == 200:
            metric_push(self.wavefront_registry, self.telegraf_client, metrics.APIMetrics.LIVE_QUERY_NOTIFICATIONS, 1)
        else:
            logger.error('Failed to send notification. Response = {}'.format(result))
            metric_push(self.wavefront_registry,self.telegraf_client, metrics.APIMetrics.LIVE_QUERY_FAILED_NOTIFICATIONS, 1)
        return result

    def prepare_data(self, base_url, admin, started_millis, status, query_name, query_id):
        """

        :param base_url: base url for live query results
        :param admin: admin who created the query
        :param started_millis: time the query was created
        :param status: status of the query
        :param query_name:
        :param query_id:
        :return:
        """
        data = [
            ('User', admin),
            ('Time', self.format_timestamp(started_millis)),
            ('Status', status)
        ]

        if query_name:
            # query name should be first
            data.insert(0, ('Query', query_name))

        template_data = {
            'base_url': base_url.rstrip('/'),
            'query_id': query_id,
            'name': query_name,
            'data': data
        }
        return template_data

    @staticmethod
    def get_email_payload(email_sender, admin, template_data, email_subject, template):
        return {
            'Source': email_sender,
            'Destination': {
                'ToAddresses': [
                    admin
                ]
            },
            'Message': {
                'Subject': {
                    'Data': email_subject
                },
                'Body': {
                    'Html': {
                        'Data': template.render(**template_data)
                    }
                }
            }
        }


def metric_push(wavefront_registry, telegraf_client, counter_name, count=1):
    if TELEGRAF_REPORTER_ENABLED:
        telegraf_counter(telegraf_client, counter_name, count)
    else:
        wavefront_counter(wavefront_registry, counter_name, count)


def wavefront_counter(wavefront_registry, counter_name, count):
    try:
        wavefront_registry.counter(counter_name).inc(count)
        logger.info('Sent wavefront metrics for %s count of 1', counter_name)
    except Exception as we:
        logger.exception("Failed to send metrics to wavefront. Error: {}".format(we))


def telegraf_counter(telegraf_client, counter_name, count=1, rate=1):
    try:
        telegraf_client.incr(counter_name, count, rate, tags={"env": ENV, "service": SERVICE_NAME, "hostname": HOSTNAME, "region": REGION})
        logger.info('Sent telegraf client metrics for %s count of 1', counter_name)
    except Exception as we:
        logger.exception("Failed to send metrics to telegraf_client. Error : {}".format(we))
